@@ -5,7 +5,7 @@
  *	have access to a console. It uses the Text widget and provides special
  *	access via a console command.
  *
- * Copyright (c) 1995-1996 Sun Microsystems, Inc.
+ * Copyright © 1995-1996 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -24,7 +24,7 @@
 typedef struct ConsoleInfo {
     Tcl_Interp *consoleInterp;	/* Interpreter displaying the console. */
     Tcl_Interp *interp;		/* Interpreter controlled by console. */
-    int refCount;
+    size_t refCount;
 } ConsoleInfo;
 
 /*
@@ -43,22 +43,21 @@ typedef struct ChannelData {
  * Prototypes for local procedures defined in this file:
  */
 
-static int	ConsoleClose(ClientData instanceData, Tcl_Interp *interp);
-static void	ConsoleDeleteProc(ClientData clientData);
-static void	ConsoleEventProc(ClientData clientData, XEvent *eventPtr);
-static int	ConsoleHandle(ClientData instanceData, int direction,
-		    ClientData *handlePtr);
-static int	ConsoleInput(ClientData instanceData, char *buf, int toRead,
+static int	ConsoleClose(void *instanceData, Tcl_Interp *interp);
+static int	Console2Close(void *instanceData, Tcl_Interp *interp, int flags);
+static void	ConsoleDeleteProc(void *clientData);
+static void	ConsoleEventProc(void *clientData, XEvent *eventPtr);
+static int	ConsoleHandle(void *instanceData, int direction,
+		    void **handlePtr);
+static int	ConsoleInput(void *instanceData, char *buf, int toRead,
 		    int *errorCode);
-static int	ConsoleObjCmd(ClientData clientData, Tcl_Interp *interp,
-		    int objc, Tcl_Obj *const objv[]);
-static int	ConsoleOutput(ClientData instanceData, const char *buf,
+static Tcl_ObjCmdProc ConsoleObjCmd;
+static int	ConsoleOutput(void *instanceData, const char *buf,
 		    int toWrite, int *errorCode);
-static void	ConsoleWatch(ClientData instanceData, int mask);
-static void	DeleteConsoleInterp(ClientData clientData);
-static void	InterpDeleteProc(ClientData clientData, Tcl_Interp *interp);
-static int	InterpreterObjCmd(ClientData clientData, Tcl_Interp *interp,
-		    int objc, Tcl_Obj *const objv[]);
+static void	ConsoleWatch(void *instanceData, int mask);
+static void	DeleteConsoleInterp(void *clientData);
+static void	InterpDeleteProc(void *clientData, Tcl_Interp *interp);
+static Tcl_ObjCmdProc InterpreterObjCmd;
 
 /*
  * This structure describes the channel type structure for file based IO:
@@ -66,8 +65,8 @@ static int	InterpreterObjCmd(ClientData clientData, Tcl_Interp *interp,
 
 static const Tcl_ChannelType consoleChannelType = {
     "console",			/* Type name. */
-    TCL_CHANNEL_VERSION_4,	/* v4 channel */
-    ConsoleClose,		/* Close proc. */
+    TCL_CHANNEL_VERSION_5,	/* v5 channel */
+    (Tcl_DriverCloseProc *)ConsoleClose,		/* Close proc. */
     ConsoleInput,		/* Input proc. */
     ConsoleOutput,		/* Output proc. */
     NULL,			/* Seek proc. */
@@ -75,7 +74,7 @@ static const Tcl_ChannelType consoleChannelType = {
     NULL,			/* Get option proc. */
     ConsoleWatch,		/* Watch for events on console. */
     ConsoleHandle,		/* Get a handle from the device. */
-    NULL,			/* close2proc. */
+    Console2Close,			/* close2proc. */
     NULL,			/* Always non-blocking.*/
     NULL,			/* flush proc. */
     NULL,			/* handler proc. */
@@ -84,7 +83,7 @@ static const Tcl_ChannelType consoleChannelType = {
     NULL
 };
 
-#ifdef __WIN32__
+#ifdef _WIN32
 #include <windows.h>
 
 /*
@@ -197,9 +196,9 @@ ShouldUseConsoleChannel(
  *
  * Tk_InitConsoleChannels --
  *
- * 	Create the console channels and install them as the standard channels.
- * 	All I/O will be discarded until Tk_CreateConsoleWindow is called to
- * 	attach the console to a text widget.
+ *	Create the console channels and install them as the standard channels.
+ *	All I/O will be discarded until Tk_CreateConsoleWindow is called to
+ *	attach the console to a text widget.
  *
  * Results:
  *	None.
@@ -220,15 +219,14 @@ Tk_InitConsoleChannels(
     Tcl_Channel consoleChannel;
 
     /*
-     * Ensure that we are getting a compatible version of Tcl. This is really
-     * only an issue when Tk is loaded dynamically.
+     * Ensure that we are getting a compatible version of Tcl.
      */
 
-    if (Tcl_InitStubs(interp, TCL_VERSION, 0) == NULL) {
-        return;
+    if (Tcl_InitStubs(interp, "9.0", 0) == NULL) {
+	return;
     }
 
-    consoleInitPtr = Tcl_GetThreadData(&consoleInitKey, (int) sizeof(int));
+    consoleInitPtr = (int *)Tcl_GetThreadData(&consoleInitKey, (int) sizeof(int));
     if (*consoleInitPtr) {
 	/*
 	 * We've already initialized console channels in this thread.
@@ -256,13 +254,13 @@ Tk_InitConsoleChannels(
      * interp for it to live in.
      */
 
-    info = ckalloc(sizeof(ConsoleInfo));
+    info = (ConsoleInfo *) ckalloc(sizeof(ConsoleInfo));
     info->consoleInterp = NULL;
     info->interp = NULL;
     info->refCount = 0;
 
     if (doIn) {
-	ChannelData *data = ckalloc(sizeof(ChannelData));
+	ChannelData *data = (ChannelData *)ckalloc(sizeof(ChannelData));
 
 	data->info = info;
 	data->info->refCount++;
@@ -279,7 +277,7 @@ Tk_InitConsoleChannels(
     }
 
     if (doOut) {
-	ChannelData *data = ckalloc(sizeof(ChannelData));
+	ChannelData *data = (ChannelData *)ckalloc(sizeof(ChannelData));
 
 	data->info = info;
 	data->info->refCount++;
@@ -296,7 +294,7 @@ Tk_InitConsoleChannels(
     }
 
     if (doErr) {
-	ChannelData *data = ckalloc(sizeof(ChannelData));
+	ChannelData *data = (ChannelData *)ckalloc(sizeof(ChannelData));
 
 	data->info = info;
 	data->info->refCount++;
@@ -319,7 +317,7 @@ Tk_InitConsoleChannels(
  * Tk_CreateConsoleWindow --
  *
  *	Initialize the console. This code actually creates a new application
- *	and associated interpreter. This effectivly hides the implementation
+ *	and associated interpreter. This effectively hides the implementation
  *	from the main application.
  *
  * Results:
@@ -345,9 +343,13 @@ Tk_CreateConsoleWindow(
     /* Init an interp with Tcl and Tk */
     Tcl_Interp *consoleInterp = Tcl_CreateInterp();
     if (Tcl_Init(consoleInterp) != TCL_OK) {
+	Tcl_Obj *result_obj = Tcl_GetObjResult(consoleInterp);
+	Tcl_SetObjResult(interp, result_obj);
 	goto error;
     }
     if (Tk_Init(consoleInterp) != TCL_OK) {
+	Tcl_Obj *result_obj = Tcl_GetObjResult(consoleInterp);
+	Tcl_SetObjResult(interp, result_obj);
 	goto error;
     }
 
@@ -374,7 +376,7 @@ Tk_CreateConsoleWindow(
 	     * New ConsoleInfo for a new console window.
 	     */
 
-	    info = ckalloc(sizeof(ConsoleInfo));
+	    info = (ConsoleInfo *)ckalloc(sizeof(ConsoleInfo));
 	    info->refCount = 0;
 
 	    /*
@@ -404,7 +406,7 @@ Tk_CreateConsoleWindow(
 	    }
 	}
     } else {
-	info = ckalloc(sizeof(ConsoleInfo));
+	info = (ConsoleInfo *)ckalloc(sizeof(ConsoleInfo));
 	info->refCount = 0;
     }
 
@@ -439,7 +441,8 @@ Tk_CreateConsoleWindow(
     }
 
     Tcl_Preserve(consoleInterp);
-    result = Tcl_GlobalEval(consoleInterp, "source $tk_library/console.tcl");
+    result = Tcl_EvalEx(consoleInterp, "source $tk_library/console.tcl",
+	    TCL_INDEX_NONE, TCL_EVAL_GLOBAL);
     if (result == TCL_ERROR) {
 	Tcl_SetReturnOptions(interp,
 		Tcl_GetReturnOptions(consoleInterp, result));
@@ -452,7 +455,7 @@ Tk_CreateConsoleWindow(
 	if (mainWindow) {
 	    Tk_DeleteEventHandler(mainWindow, StructureNotifyMask,
 		    ConsoleEventProc, info);
-	    if (--info->refCount <= 0) {
+	    if (info->refCount-- <= 1) {
 		ckfree(info);
 	    }
 	}
@@ -488,12 +491,12 @@ Tk_CreateConsoleWindow(
 
 static int
 ConsoleOutput(
-    ClientData instanceData,	/* Indicates which device to use. */
+    void *instanceData,	/* Indicates which device to use. */
     const char *buf,		/* The data buffer. */
     int toWrite,		/* How many bytes to write? */
     int *errorCode)		/* Where to store error code. */
 {
-    ChannelData *data = instanceData;
+    ChannelData *data = (ChannelData *)instanceData;
     ConsoleInfo *info = data->info;
 
     *errorCode = 0;
@@ -512,26 +515,25 @@ ConsoleOutput(
 	     * Assumption is utf-8 Tcl_Encoding is reliably present.
 	     */
 
-	    const char *bytes
-		    = Tcl_ExternalToUtfDString(utf8, buf, toWrite, &ds);
-	    int numBytes = Tcl_DStringLength(&ds);
-	    Tcl_Obj *cmd = Tcl_NewStringObj("tk::ConsoleOutput", -1);
+	    const char *bytes = Tcl_ExternalToUtfDString(utf8, buf, toWrite, &ds);
+	    Tcl_Size numBytes = Tcl_DStringLength(&ds);
+	    Tcl_Obj *cmd = Tcl_NewStringObj("tk::ConsoleOutput", TCL_INDEX_NONE);
 
 	    Tcl_FreeEncoding(utf8);
 
 	    if (data->type == TCL_STDERR) {
 		Tcl_ListObjAppendElement(NULL, cmd,
-			Tcl_NewStringObj("stderr", -1));
+			Tcl_NewStringObj("stderr", TCL_INDEX_NONE));
 	    } else {
 		Tcl_ListObjAppendElement(NULL, cmd,
-			Tcl_NewStringObj("stdout", -1));
+			Tcl_NewStringObj("stdout", TCL_INDEX_NONE));
 	    }
 	    Tcl_ListObjAppendElement(NULL, cmd,
 		    Tcl_NewStringObj(bytes, numBytes));
 
 	    Tcl_DStringFree(&ds);
 	    Tcl_IncrRefCount(cmd);
-	    Tcl_GlobalEvalObj(consoleInterp, cmd);
+	    Tcl_EvalObjEx(consoleInterp, cmd, TCL_EVAL_GLOBAL);
 	    Tcl_DecrRefCount(cmd);
 	}
     }
@@ -554,14 +556,13 @@ ConsoleOutput(
  *----------------------------------------------------------------------
  */
 
-	/* ARGSUSED */
 static int
 ConsoleInput(
-    ClientData instanceData,	/* Unused. */
-    char *buf,			/* Where to store data read. */
-    int bufSize,		/* How much space is available in the
+    TCL_UNUSED(void *),
+    TCL_UNUSED(char *),			/* Where to store data read. */
+    TCL_UNUSED(int),		/* How much space is available in the
 				 * buffer? */
-    int *errorCode)		/* Where to store error code. */
+    TCL_UNUSED(int *))		/* Where to store error code. */
 {
     return 0;			/* Always return EOF. */
 }
@@ -569,7 +570,7 @@ ConsoleInput(
 /*
  *----------------------------------------------------------------------
  *
- * ConsoleClose --
+ * ConsoleClose/Console2Close --
  *
  *	Closes the IO channel.
  *
@@ -582,17 +583,16 @@ ConsoleInput(
  *----------------------------------------------------------------------
  */
 
-	/* ARGSUSED */
 static int
 ConsoleClose(
-    ClientData instanceData,	/* Unused. */
-    Tcl_Interp *interp)		/* Unused. */
+    void *instanceData,
+    TCL_UNUSED(Tcl_Interp *))
 {
-    ChannelData *data = instanceData;
+    ChannelData *data = (ChannelData *)instanceData;
     ConsoleInfo *info = data->info;
 
     if (info) {
-	if (--info->refCount <= 0) {
+	if (info->refCount-- <= 1) {
 	    /*
 	     * Assuming the Tcl_Interp * fields must already be NULL.
 	     */
@@ -603,6 +603,18 @@ ConsoleClose(
     ckfree(data);
     return 0;
 }
+
+static int
+Console2Close(
+    void *instanceData,	/* Unused. */
+    Tcl_Interp *interp,		/* Unused. */
+    int flags)
+{
+    if ((flags&(TCL_CLOSE_READ|TCL_CLOSE_WRITE))==0) {
+	return ConsoleClose(instanceData, interp);
+    }
+    return EINVAL;
+}
 
 /*
  *----------------------------------------------------------------------
@@ -622,11 +634,10 @@ ConsoleClose(
  *----------------------------------------------------------------------
  */
 
-	/* ARGSUSED */
 static void
 ConsoleWatch(
-    ClientData instanceData,	/* Device ID for the channel. */
-    int mask)			/* OR-ed combination of TCL_READABLE,
+    TCL_UNUSED(void *),	/* Device ID for the channel. */
+    TCL_UNUSED(int))			/* OR-ed combination of TCL_READABLE,
 				 * TCL_WRITABLE and TCL_EXCEPTION, for the
 				 * events we are interested in. */
 {
@@ -649,14 +660,13 @@ ConsoleWatch(
  *----------------------------------------------------------------------
  */
 
-	/* ARGSUSED */
 static int
 ConsoleHandle(
-    ClientData instanceData,	/* Device ID for the channel. */
-    int direction,		/* TCL_READABLE or TCL_WRITABLE to indicate
+    TCL_UNUSED(void *),	/* Device ID for the channel. */
+    TCL_UNUSED(int),		/* TCL_READABLE or TCL_WRITABLE to indicate
 				 * which direction of the channel is being
 				 * requested. */
-    ClientData *handlePtr)	/* Where to store handle */
+    TCL_UNUSED(void **))	/* Where to store handle */
 {
     return TCL_ERROR;
 }
@@ -680,7 +690,7 @@ ConsoleHandle(
 
 static int
 ConsoleObjCmd(
-    ClientData clientData,	/* Access to the console interp */
+    void *clientData,	/* Access to the console interp */
     Tcl_Interp *interp,		/* Current interpreter */
     int objc,			/* Number of arguments */
     Tcl_Obj *const objv[])	/* Argument objects */
@@ -690,15 +700,15 @@ ConsoleObjCmd(
 	"eval", "hide", "show", "title", NULL};
     enum option {CON_EVAL, CON_HIDE, CON_SHOW, CON_TITLE};
     Tcl_Obj *cmd = NULL;
-    ConsoleInfo *info = clientData;
+    ConsoleInfo *info = (ConsoleInfo *)clientData;
     Tcl_Interp *consoleInterp = info->consoleInterp;
 
     if (objc < 2) {
 	Tcl_WrongNumArgs(interp, 1, objv, "option ?arg?");
 	return TCL_ERROR;
     }
-    if (Tcl_GetIndexFromObj(interp, objv[1], options, "option", 0, &index)
-	    != TCL_OK) {
+    if (Tcl_GetIndexFromObjStruct(interp, objv[1], options,
+	    sizeof(char *), "option", 0, &index) != TCL_OK) {
 	return TCL_ERROR;
     }
 
@@ -715,21 +725,21 @@ ConsoleObjCmd(
 	    Tcl_WrongNumArgs(interp, 2, objv, NULL);
 	    return TCL_ERROR;
 	}
-	cmd = Tcl_NewStringObj("wm withdraw .", -1);
+	cmd = Tcl_NewStringObj("wm withdraw .", TCL_INDEX_NONE);
 	break;
     case CON_SHOW:
 	if (objc != 2) {
 	    Tcl_WrongNumArgs(interp, 2, objv, NULL);
 	    return TCL_ERROR;
 	}
-	cmd = Tcl_NewStringObj("wm deiconify .", -1);
+	cmd = Tcl_NewStringObj("wm deiconify .", TCL_INDEX_NONE);
 	break;
     case CON_TITLE:
 	if (objc > 3) {
 	    Tcl_WrongNumArgs(interp, 2, objv, "?title?");
 	    return TCL_ERROR;
 	}
-	cmd = Tcl_NewStringObj("wm title .", -1);
+	cmd = Tcl_NewStringObj("wm title .", TCL_INDEX_NONE);
 	if (objc == 3) {
 	    Tcl_ListObjAppendElement(NULL, cmd, objv[2]);
 	}
@@ -741,14 +751,14 @@ ConsoleObjCmd(
     Tcl_IncrRefCount(cmd);
     if (consoleInterp && !Tcl_InterpDeleted(consoleInterp)) {
 	Tcl_Preserve(consoleInterp);
-	result = Tcl_GlobalEvalObj(consoleInterp, cmd);
+	result = Tcl_EvalObjEx(consoleInterp, cmd, TCL_EVAL_GLOBAL);
 	Tcl_SetReturnOptions(interp,
 		Tcl_GetReturnOptions(consoleInterp, result));
 	Tcl_SetObjResult(interp, Tcl_GetObjResult(consoleInterp));
 	Tcl_Release(consoleInterp);
     } else {
 	Tcl_SetObjResult(interp, Tcl_NewStringObj(
-		"no active console interp", -1));
+		"no active console interp", TCL_INDEX_NONE));
 	Tcl_SetErrorCode(interp, "TK", "CONSOLE", "NONE", NULL);
 	result = TCL_ERROR;
     }
@@ -772,7 +782,7 @@ ConsoleObjCmd(
 
 static int
 InterpreterObjCmd(
-    ClientData clientData,	/* */
+    void *clientData,	/* */
     Tcl_Interp *interp,		/* Current interpreter */
     int objc,			/* Number of arguments */
     Tcl_Obj *const objv[])	/* Argument objects */
@@ -780,15 +790,15 @@ InterpreterObjCmd(
     int index, result = TCL_OK;
     static const char *const options[] = {"eval", "record", NULL};
     enum option {OTHER_EVAL, OTHER_RECORD};
-    ConsoleInfo *info = clientData;
+    ConsoleInfo *info = (ConsoleInfo *)clientData;
     Tcl_Interp *otherInterp = info->interp;
 
     if (objc < 2) {
 	Tcl_WrongNumArgs(interp, 1, objv, "option arg");
 	return TCL_ERROR;
     }
-    if (Tcl_GetIndexFromObj(interp, objv[1], options, "option", 0, &index)
-	    != TCL_OK) {
+    if (Tcl_GetIndexFromObjStruct(interp, objv[1], options,
+	    sizeof(char *), "option", 0, &index) != TCL_OK) {
 	return TCL_ERROR;
     }
 
@@ -799,7 +809,7 @@ InterpreterObjCmd(
 
     if ((otherInterp == NULL) || Tcl_InterpDeleted(otherInterp)) {
 	Tcl_SetObjResult(interp, Tcl_NewStringObj(
-		"no active master interp", -1));
+		"no active parent interp", TCL_INDEX_NONE));
 	Tcl_SetErrorCode(interp, "TK", "CONSOLE", "NO_INTERP", NULL);
 	return TCL_ERROR;
     }
@@ -807,7 +817,7 @@ InterpreterObjCmd(
     Tcl_Preserve(otherInterp);
     switch ((enum option) index) {
     case OTHER_EVAL:
-   	result = Tcl_GlobalEvalObj(otherInterp, objv[2]);
+	result = Tcl_EvalObjEx(otherInterp, objv[2], TCL_EVAL_GLOBAL);
 
 	/*
 	 * TODO: Should exceptions be filtered here?
@@ -818,7 +828,7 @@ InterpreterObjCmd(
 	Tcl_SetObjResult(interp, Tcl_GetObjResult(otherInterp));
 	break;
     case OTHER_RECORD:
-   	Tcl_RecordAndEvalObj(otherInterp, objv[2], TCL_EVAL_GLOBAL);
+	Tcl_RecordAndEvalObj(otherInterp, objv[2], TCL_EVAL_GLOBAL);
 
 	/*
 	 * By not setting result, we discard any exceptions or errors here and
@@ -846,9 +856,9 @@ InterpreterObjCmd(
 
 static void
 DeleteConsoleInterp(
-    ClientData clientData)
+    void *clientData)
 {
-    Tcl_Interp *interp = clientData;
+    Tcl_Interp *interp = (Tcl_Interp *)clientData;
 
     Tcl_DeleteInterp(interp);
 }
@@ -872,16 +882,16 @@ DeleteConsoleInterp(
 
 static void
 InterpDeleteProc(
-    ClientData clientData,
+    void *clientData,
     Tcl_Interp *interp)
 {
-    ConsoleInfo *info = clientData;
+    ConsoleInfo *info = (ConsoleInfo *)clientData;
 
     if (info->consoleInterp == interp) {
 	Tcl_DeleteThreadExitHandler(DeleteConsoleInterp, info->consoleInterp);
 	info->consoleInterp = NULL;
     }
-    if (--info->refCount <= 0) {
+    if (info->refCount-- <= 1) {
 	ckfree(info);
     }
 }
@@ -905,14 +915,14 @@ InterpDeleteProc(
 
 static void
 ConsoleDeleteProc(
-    ClientData clientData)
+    void *clientData)
 {
-    ConsoleInfo *info = clientData;
+    ConsoleInfo *info = (ConsoleInfo *)clientData;
 
     if (info->consoleInterp) {
 	Tcl_DeleteInterp(info->consoleInterp);
     }
-    if (--info->refCount <= 0) {
+    if (info->refCount-- <= 1) {
 	ckfree(info);
     }
 }
@@ -922,7 +932,7 @@ ConsoleDeleteProc(
  *
  * ConsoleEventProc --
  *
- *	This event function is registered on the main window of the slave
+ *	This event function is registered on the main window of the child
  *	interpreter. If the user or a running script causes the main window to
  *	be destroyed, then we need to inform the console interpreter by
  *	invoking "::tk::ConsoleExit".
@@ -938,18 +948,18 @@ ConsoleDeleteProc(
 
 static void
 ConsoleEventProc(
-    ClientData clientData,
+    void *clientData,
     XEvent *eventPtr)
 {
     if (eventPtr->type == DestroyNotify) {
-	ConsoleInfo *info = clientData;
+	ConsoleInfo *info = (ConsoleInfo *)clientData;
 	Tcl_Interp *consoleInterp = info->consoleInterp;
 
 	if (consoleInterp && !Tcl_InterpDeleted(consoleInterp)) {
-	    Tcl_GlobalEval(consoleInterp, "tk::ConsoleExit");
+	    Tcl_EvalEx(consoleInterp, "tk::ConsoleExit", TCL_INDEX_NONE, TCL_EVAL_GLOBAL);
 	}
 
-	if (--info->refCount <= 0) {
+	if (info->refCount-- <= 1) {
 	    ckfree(info);
 	}
     }
